// Copyright 2014 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_

#include <stddef.h>

#include "base/check_op.h"
#include "third_party/blink/public/mojom/use_counter/metrics/css_property_id.mojom-blink-forward.h"
#include "third_party/blink/renderer/core/core_export.h"
#include "third_party/blink/renderer/platform/wtf/allocator/allocator.h"
#include "third_party/blink/renderer/platform/wtf/hash_functions.h"
#include "third_party/blink/renderer/platform/wtf/hash_traits.h"

namespace WTF {
class AtomicString;
class String;
}  // namespace WTF

namespace blink {

class ExecutionContext;

enum class CSSPropertyID {
    kInvalid = 0,
    kVariable = 1,
    kColor = 2,
    kDirection = 3,
    kFontFamily = 4,
    kFontFeatureSettings = 5,
    kFontKerning = 6,
    kFontOpticalSizing = 7,
    kFontPalette = 8,
    kFontSize = 9,
    kFontSizeAdjust = 10,
    kFontStretch = 11,
    kFontStyle = 12,
    kFontSynthesisSmallCaps = 13,
    kFontSynthesisStyle = 14,
    kFontSynthesisWeight = 15,
    kFontVariantAlternates = 16,
    kFontVariantCaps = 17,
    kFontVariantEastAsian = 18,
    kFontVariantLigatures = 19,
    kFontVariantNumeric = 20,
    kFontVariantPosition = 21,
    kFontVariationSettings = 22,
    kFontWeight = 23,
    kForcedColorAdjust = 24,
    kInternalVisitedColor = 25,
    kTextOrientation = 26,
    kTextRendering = 27,
    kTextSpacingTrim = 28,
    kWebkitFontSmoothing = 29,
    kWebkitLocale = 30,
    kWebkitTextOrientation = 31,
    kWebkitWritingMode = 32,
    kWritingMode = 33,
    kZoom = 34,
    kAccentColor = 35,
    kAdditiveSymbols = 36,
    kAlignContent = 37,
    kAlignItems = 38,
    kAlignSelf = 39,
    kAlignmentBaseline = 40,
    kAll = 41,
    kAnchorDefault = 42,
    kAnchorName = 43,
    kAnimationComposition = 44,
    kAnimationDelay = 45,
    kAnimationDelayEnd = 46,
    kAnimationDelayStart = 47,
    kAnimationDirection = 48,
    kAnimationDuration = 49,
    kAnimationFillMode = 50,
    kAnimationIterationCount = 51,
    kAnimationName = 52,
    kAnimationPlayState = 53,
    kAnimationRangeEnd = 54,
    kAnimationRangeStart = 55,
    kAnimationTimeline = 56,
    kAnimationTimingFunction = 57,
    kAppRegion = 58,
    kAppearance = 59,
    kAscentOverride = 60,
    kAspectRatio = 61,
    kBackdropFilter = 62,
    kBackfaceVisibility = 63,
    kBackgroundAttachment = 64,
    kBackgroundBlendMode = 65,
    kBackgroundClip = 66,
    kBackgroundColor = 67,
    kBackgroundImage = 68,
    kBackgroundOrigin = 69,
    kBackgroundPositionX = 70,
    kBackgroundPositionY = 71,
    kBackgroundRepeat = 72,
    kBackgroundSize = 73,
    kBasePalette = 74,
    kBaselineShift = 75,
    kBaselineSource = 76,
    kBlockSize = 77,
    kBorderBlockEndColor = 78,
    kBorderBlockEndStyle = 79,
    kBorderBlockEndWidth = 80,
    kBorderBlockStartColor = 81,
    kBorderBlockStartStyle = 82,
    kBorderBlockStartWidth = 83,
    kBorderBottomColor = 84,
    kBorderBottomLeftRadius = 85,
    kBorderBottomRightRadius = 86,
    kBorderBottomStyle = 87,
    kBorderBottomWidth = 88,
    kBorderCollapse = 89,
    kBorderEndEndRadius = 90,
    kBorderEndStartRadius = 91,
    kBorderImageOutset = 92,
    kBorderImageRepeat = 93,
    kBorderImageSlice = 94,
    kBorderImageSource = 95,
    kBorderImageWidth = 96,
    kBorderInlineEndColor = 97,
    kBorderInlineEndStyle = 98,
    kBorderInlineEndWidth = 99,
    kBorderInlineStartColor = 100,
    kBorderInlineStartStyle = 101,
    kBorderInlineStartWidth = 102,
    kBorderLeftColor = 103,
    kBorderLeftStyle = 104,
    kBorderLeftWidth = 105,
    kBorderRightColor = 106,
    kBorderRightStyle = 107,
    kBorderRightWidth = 108,
    kBorderStartEndRadius = 109,
    kBorderStartStartRadius = 110,
    kBorderTopColor = 111,
    kBorderTopLeftRadius = 112,
    kBorderTopRightRadius = 113,
    kBorderTopStyle = 114,
    kBorderTopWidth = 115,
    kBottom = 116,
    kBoxShadow = 117,
    kBoxSizing = 118,
    kBreakAfter = 119,
    kBreakBefore = 120,
    kBreakInside = 121,
    kBufferedRendering = 122,
    kCaptionSide = 123,
    kCaretColor = 124,
    kClear = 125,
    kClip = 126,
    kClipPath = 127,
    kClipRule = 128,
    kColorInterpolation = 129,
    kColorInterpolationFilters = 130,
    kColorRendering = 131,
    kColorScheme = 132,
    kColumnCount = 133,
    kColumnFill = 134,
    kColumnGap = 135,
    kColumnRuleColor = 136,
    kColumnRuleStyle = 137,
    kColumnRuleWidth = 138,
    kColumnSpan = 139,
    kColumnWidth = 140,
    kContain = 141,
    kContainIntrinsicBlockSize = 142,
    kContainIntrinsicHeight = 143,
    kContainIntrinsicInlineSize = 144,
    kContainIntrinsicWidth = 145,
    kContainerName = 146,
    kContainerType = 147,
    kContent = 148,
    kContentVisibility = 149,
    kCounterIncrement = 150,
    kCounterReset = 151,
    kCounterSet = 152,
    kCursor = 153,
    kCx = 154,
    kCy = 155,
    kD = 156,
    kDescentOverride = 157,
    kDisplay = 158,
    kDominantBaseline = 159,
    kDynamicRangeLimit = 160,
    kEmptyCells = 161,
    kFallback = 162,
    kFieldSizing = 163,
    kFill = 164,
    kFillOpacity = 165,
    kFillRule = 166,
    kFilter = 167,
    kFlexBasis = 168,
    kFlexDirection = 169,
    kFlexGrow = 170,
    kFlexShrink = 171,
    kFlexWrap = 172,
    kFloat = 173,
    kFloodColor = 174,
    kFloodOpacity = 175,
    kFontDisplay = 176,
    kGridAutoColumns = 177,
    kGridAutoFlow = 178,
    kGridAutoRows = 179,
    kGridColumnEnd = 180,
    kGridColumnStart = 181,
    kGridRowEnd = 182,
    kGridRowStart = 183,
    kGridTemplateAreas = 184,
    kGridTemplateColumns = 185,
    kGridTemplateRows = 186,
    kHeight = 187,
    kHyphenateCharacter = 188,
    kHyphenateLimitChars = 189,
    kHyphens = 190,
    kImageOrientation = 191,
    kImageRendering = 192,
    kInherits = 193,
    kInitialLetter = 194,
    kInitialValue = 195,
    kInlineSize = 196,
    kInsetBlockEnd = 197,
    kInsetBlockStart = 198,
    kInsetInlineEnd = 199,
    kInsetInlineStart = 200,
    kInternalAlignContentBlock = 201,
    kInternalEmptyLineHeight = 202,
    kInternalFontSizeDelta = 203,
    kInternalForcedBackgroundColor = 204,
    kInternalForcedBorderColor = 205,
    kInternalForcedColor = 206,
    kInternalForcedOutlineColor = 207,
    kInternalForcedVisitedColor = 208,
    kInternalOverflowBlock = 209,
    kInternalOverflowInline = 210,
    kInternalVisitedBackgroundColor = 211,
    kInternalVisitedBorderBlockEndColor = 212,
    kInternalVisitedBorderBlockStartColor = 213,
    kInternalVisitedBorderBottomColor = 214,
    kInternalVisitedBorderInlineEndColor = 215,
    kInternalVisitedBorderInlineStartColor = 216,
    kInternalVisitedBorderLeftColor = 217,
    kInternalVisitedBorderRightColor = 218,
    kInternalVisitedBorderTopColor = 219,
    kInternalVisitedCaretColor = 220,
    kInternalVisitedColumnRuleColor = 221,
    kInternalVisitedFill = 222,
    kInternalVisitedOutlineColor = 223,
    kInternalVisitedStroke = 224,
    kInternalVisitedTextDecorationColor = 225,
    kInternalVisitedTextEmphasisColor = 226,
    kInternalVisitedTextFillColor = 227,
    kInternalVisitedTextStrokeColor = 228,
    kIsolation = 229,
    kJustifyContent = 230,
    kJustifyItems = 231,
    kJustifySelf = 232,
    kLeft = 233,
    kLetterSpacing = 234,
    kLightingColor = 235,
    kLineBreak = 236,
    kLineGapOverride = 237,
    kLineHeight = 238,
    kListStyleImage = 239,
    kListStylePosition = 240,
    kListStyleType = 241,
    kMarginBlockEnd = 242,
    kMarginBlockStart = 243,
    kMarginBottom = 244,
    kMarginInlineEnd = 245,
    kMarginInlineStart = 246,
    kMarginLeft = 247,
    kMarginRight = 248,
    kMarginTop = 249,
    kMarkerEnd = 250,
    kMarkerMid = 251,
    kMarkerStart = 252,
    kMask = 253,
    kMaskClip = 254,
    kMaskComposite = 255,
    kMaskImage = 256,
    kMaskMode = 257,
    kMaskOrigin = 258,
    kMaskRepeat = 259,
    kMaskSize = 260,
    kMaskType = 261,
    kMathDepth = 262,
    kMathShift = 263,
    kMathStyle = 264,
    kMaxBlockSize = 265,
    kMaxHeight = 266,
    kMaxInlineSize = 267,
    kMaxWidth = 268,
    kMinBlockSize = 269,
    kMinHeight = 270,
    kMinInlineSize = 271,
    kMinWidth = 272,
    kMixBlendMode = 273,
    kNavigationTrigger = 274,
    kNegative = 275,
    kObjectFit = 276,
    kObjectPosition = 277,
    kObjectViewBox = 278,
    kOffsetAnchor = 279,
    kOffsetDistance = 280,
    kOffsetPath = 281,
    kOffsetPosition = 282,
    kOffsetRotate = 283,
    kOpacity = 284,
    kOrder = 285,
    kOriginTrialTestProperty = 286,
    kOrphans = 287,
    kOutlineColor = 288,
    kOutlineOffset = 289,
    kOutlineStyle = 290,
    kOutlineWidth = 291,
    kOverflowAnchor = 292,
    kOverflowBlock = 293,
    kOverflowClipMargin = 294,
    kOverflowInline = 295,
    kOverflowWrap = 296,
    kOverflowX = 297,
    kOverflowY = 298,
    kOverlay = 299,
    kOverrideColors = 300,
    kOverscrollBehaviorBlock = 301,
    kOverscrollBehaviorInline = 302,
    kOverscrollBehaviorX = 303,
    kOverscrollBehaviorY = 304,
    kPad = 305,
    kPaddingBlockEnd = 306,
    kPaddingBlockStart = 307,
    kPaddingBottom = 308,
    kPaddingInlineEnd = 309,
    kPaddingInlineStart = 310,
    kPaddingLeft = 311,
    kPaddingRight = 312,
    kPaddingTop = 313,
    kPage = 314,
    kPageOrientation = 315,
    kPaintOrder = 316,
    kPerspective = 317,
    kPerspectiveOrigin = 318,
    kPointerEvents = 319,
    kPopoverHideDelay = 320,
    kPopoverShowDelay = 321,
    kPosition = 322,
    kPositionFallback = 323,
    kPositionFallbackBounds = 324,
    kPrefix = 325,
    kQuotes = 326,
    kR = 327,
    kRange = 328,
    kResize = 329,
    kRight = 330,
    kRotate = 331,
    kRowGap = 332,
    kRubyPosition = 333,
    kRx = 334,
    kRy = 335,
    kScale = 336,
    kScrollBehavior = 337,
    kScrollMarginBlockEnd = 338,
    kScrollMarginBlockStart = 339,
    kScrollMarginBottom = 340,
    kScrollMarginInlineEnd = 341,
    kScrollMarginInlineStart = 342,
    kScrollMarginLeft = 343,
    kScrollMarginRight = 344,
    kScrollMarginTop = 345,
    kScrollPaddingBlockEnd = 346,
    kScrollPaddingBlockStart = 347,
    kScrollPaddingBottom = 348,
    kScrollPaddingInlineEnd = 349,
    kScrollPaddingInlineStart = 350,
    kScrollPaddingLeft = 351,
    kScrollPaddingRight = 352,
    kScrollPaddingTop = 353,
    kScrollSnapAlign = 354,
    kScrollSnapStop = 355,
    kScrollSnapType = 356,
    kScrollStartBlock = 357,
    kScrollStartInline = 358,
    kScrollStartTargetBlock = 359,
    kScrollStartTargetInline = 360,
    kScrollStartTargetX = 361,
    kScrollStartTargetY = 362,
    kScrollStartX = 363,
    kScrollStartY = 364,
    kScrollTimelineAxis = 365,
    kScrollTimelineName = 366,
    kScrollbarColor = 367,
    kScrollbarGutter = 368,
    kScrollbarWidth = 369,
    kShapeImageThreshold = 370,
    kShapeMargin = 371,
    kShapeOutside = 372,
    kShapeRendering = 373,
    kSize = 374,
    kSizeAdjust = 375,
    kSpeak = 376,
    kSpeakAs = 377,
    kSrc = 378,
    kStopColor = 379,
    kStopOpacity = 380,
    kStroke = 381,
    kStrokeDasharray = 382,
    kStrokeDashoffset = 383,
    kStrokeLinecap = 384,
    kStrokeLinejoin = 385,
    kStrokeMiterlimit = 386,
    kStrokeOpacity = 387,
    kStrokeWidth = 388,
    kSuffix = 389,
    kSymbols = 390,
    kSyntax = 391,
    kSystem = 392,
    kTabSize = 393,
    kTableLayout = 394,
    kTextAlign = 395,
    kTextAlignLast = 396,
    kTextAnchor = 397,
    kTextAutospace = 398,
    kTextBoxTrim = 399,
    kTextCombineUpright = 400,
    kTextDecorationColor = 401,
    kTextDecorationLine = 402,
    kTextDecorationSkipInk = 403,
    kTextDecorationStyle = 404,
    kTextDecorationThickness = 405,
    kTextEmphasisColor = 406,
    kTextEmphasisPosition = 407,
    kTextEmphasisStyle = 408,
    kTextIndent = 409,
    kTextOverflow = 410,
    kTextShadow = 411,
    kTextSizeAdjust = 412,
    kTextTransform = 413,
    kTextUnderlineOffset = 414,
    kTextUnderlinePosition = 415,
    kTextWrap = 416,
    kTimelineScope = 417,
    kTop = 418,
    kTouchAction = 419,
    kTransform = 420,
    kTransformBox = 421,
    kTransformOrigin = 422,
    kTransformStyle = 423,
    kTransitionBehavior = 424,
    kTransitionDelay = 425,
    kTransitionDuration = 426,
    kTransitionProperty = 427,
    kTransitionTimingFunction = 428,
    kTranslate = 429,
    kUnicodeBidi = 430,
    kUnicodeRange = 431,
    kUserSelect = 432,
    kVectorEffect = 433,
    kVerticalAlign = 434,
    kViewTimelineAxis = 435,
    kViewTimelineInset = 436,
    kViewTimelineName = 437,
    kViewTransitionName = 438,
    kVisibility = 439,
    kWebkitBorderHorizontalSpacing = 440,
    kWebkitBorderImage = 441,
    kWebkitBorderVerticalSpacing = 442,
    kWebkitBoxAlign = 443,
    kWebkitBoxDecorationBreak = 444,
    kWebkitBoxDirection = 445,
    kWebkitBoxDirectionAlternative = 446,
    kWebkitBoxFlex = 447,
    kWebkitBoxOrdinalGroup = 448,
    kWebkitBoxOrient = 449,
    kWebkitBoxPack = 450,
    kWebkitBoxReflect = 451,
    kWebkitLineBreak = 452,
    kWebkitLineClamp = 453,
    kWebkitMaskBoxImageOutset = 454,
    kWebkitMaskBoxImageRepeat = 455,
    kWebkitMaskBoxImageSlice = 456,
    kWebkitMaskBoxImageSource = 457,
    kWebkitMaskBoxImageWidth = 458,
    kWebkitMaskClip = 459,
    kWebkitMaskComposite = 460,
    kWebkitMaskImage = 461,
    kWebkitMaskOrigin = 462,
    kWebkitMaskPositionX = 463,
    kWebkitMaskPositionY = 464,
    kWebkitMaskRepeat = 465,
    kWebkitMaskSize = 466,
    kWebkitPerspectiveOriginX = 467,
    kWebkitPerspectiveOriginY = 468,
    kWebkitPrintColorAdjust = 469,
    kWebkitRtlOrdering = 470,
    kWebkitRubyPosition = 471,
    kWebkitTapHighlightColor = 472,
    kWebkitTextCombine = 473,
    kWebkitTextDecorationsInEffect = 474,
    kWebkitTextFillColor = 475,
    kWebkitTextSecurity = 476,
    kWebkitTextStrokeColor = 477,
    kWebkitTextStrokeWidth = 478,
    kWebkitTransformOriginX = 479,
    kWebkitTransformOriginY = 480,
    kWebkitTransformOriginZ = 481,
    kWebkitUserDrag = 482,
    kWebkitUserModify = 483,
    kWhiteSpaceCollapse = 484,
    kWidows = 485,
    kWidth = 486,
    kWillChange = 487,
    kWordBreak = 488,
    kWordSpacing = 489,
    kX = 490,
    kY = 491,
    kZIndex = 492,
    kAlternativeAnimationDelay = 493,
    kAlternativeAnimationWithDelayStartEnd = 494,
    kAlternativeAnimationWithTimeline = 495,
    kAlternativeMask = 496,
    kAlternativeViewTimelineWithInset = 497,
    kAnimation = 498,
    kAnimationRange = 499,
    kBackground = 500,
    kBackgroundPosition = 501,
    kBorder = 502,
    kBorderBlock = 503,
    kBorderBlockColor = 504,
    kBorderBlockEnd = 505,
    kBorderBlockStart = 506,
    kBorderBlockStyle = 507,
    kBorderBlockWidth = 508,
    kBorderBottom = 509,
    kBorderColor = 510,
    kBorderImage = 511,
    kBorderInline = 512,
    kBorderInlineColor = 513,
    kBorderInlineEnd = 514,
    kBorderInlineStart = 515,
    kBorderInlineStyle = 516,
    kBorderInlineWidth = 517,
    kBorderLeft = 518,
    kBorderRadius = 519,
    kBorderRight = 520,
    kBorderSpacing = 521,
    kBorderStyle = 522,
    kBorderTop = 523,
    kBorderWidth = 524,
    kColumnRule = 525,
    kColumns = 526,
    kContainIntrinsicSize = 527,
    kContainer = 528,
    kFlex = 529,
    kFlexFlow = 530,
    kFont = 531,
    kFontSynthesis = 532,
    kFontVariant = 533,
    kGap = 534,
    kGrid = 535,
    kGridArea = 536,
    kGridColumn = 537,
    kGridColumnGap = 538,
    kGridGap = 539,
    kGridRow = 540,
    kGridRowGap = 541,
    kGridTemplate = 542,
    kInset = 543,
    kInsetBlock = 544,
    kInsetInline = 545,
    kListStyle = 546,
    kMargin = 547,
    kMarginBlock = 548,
    kMarginInline = 549,
    kMarker = 550,
    kMaskPosition = 551,
    kOffset = 552,
    kOutline = 553,
    kOverflow = 554,
    kOverscrollBehavior = 555,
    kPadding = 556,
    kPaddingBlock = 557,
    kPaddingInline = 558,
    kPageBreakAfter = 559,
    kPageBreakBefore = 560,
    kPageBreakInside = 561,
    kPlaceContent = 562,
    kPlaceItems = 563,
    kPlaceSelf = 564,
    kScrollMargin = 565,
    kScrollMarginBlock = 566,
    kScrollMarginInline = 567,
    kScrollPadding = 568,
    kScrollPaddingBlock = 569,
    kScrollPaddingInline = 570,
    kScrollStart = 571,
    kScrollStartTarget = 572,
    kScrollTimeline = 573,
    kTextDecoration = 574,
    kTextEmphasis = 575,
    kTextSpacing = 576,
    kTransition = 577,
    kViewTimeline = 578,
    kWebkitColumnBreakAfter = 579,
    kWebkitColumnBreakBefore = 580,
    kWebkitColumnBreakInside = 581,
    kWebkitMask = 582,
    kWebkitMaskBoxImage = 583,
    kWebkitMaskPosition = 584,
    kWebkitTextStroke = 585,
    kWhiteSpace = 586,
    kAliasWebkitAppearance = 587,
    kAliasWebkitAppRegion = 588,
    kAliasWebkitAlternativeMaskClip = 589,
    kAliasWebkitAlternativeMaskComposite = 590,
    kAliasWebkitAlternativeMaskImage = 591,
    kAliasWebkitAlternativeMaskOrigin = 592,
    kAliasWebkitAlternativeMaskRepeat = 593,
    kAliasWebkitAlternativeMaskSize = 594,
    kAliasWebkitBorderEndColor = 595,
    kAliasWebkitBorderEndStyle = 596,
    kAliasWebkitBorderEndWidth = 597,
    kAliasWebkitBorderStartColor = 598,
    kAliasWebkitBorderStartStyle = 599,
    kAliasWebkitBorderStartWidth = 600,
    kAliasWebkitBorderBeforeColor = 601,
    kAliasWebkitBorderBeforeStyle = 602,
    kAliasWebkitBorderBeforeWidth = 603,
    kAliasWebkitBorderAfterColor = 604,
    kAliasWebkitBorderAfterStyle = 605,
    kAliasWebkitBorderAfterWidth = 606,
    kAliasWebkitMarginEnd = 607,
    kAliasWebkitMarginStart = 608,
    kAliasWebkitMarginBefore = 609,
    kAliasWebkitMarginAfter = 610,
    kAliasWebkitPaddingEnd = 611,
    kAliasWebkitPaddingStart = 612,
    kAliasWebkitPaddingBefore = 613,
    kAliasWebkitPaddingAfter = 614,
    kAliasWebkitLogicalWidth = 615,
    kAliasWebkitLogicalHeight = 616,
    kAliasWebkitMinLogicalWidth = 617,
    kAliasWebkitMinLogicalHeight = 618,
    kAliasWebkitMaxLogicalWidth = 619,
    kAliasWebkitMaxLogicalHeight = 620,
    kAliasWebkitBorderAfter = 621,
    kAliasWebkitBorderBefore = 622,
    kAliasWebkitBorderEnd = 623,
    kAliasWebkitBorderStart = 624,
    kAliasWebkitAlternativeMask = 625,
    kAliasWebkitAlternativeMaskPosition = 626,
    kAliasEpubCaptionSide = 627,
    kAliasEpubTextCombine = 628,
    kAliasEpubTextEmphasis = 629,
    kAliasEpubTextEmphasisColor = 630,
    kAliasEpubTextEmphasisStyle = 631,
    kAliasEpubTextOrientation = 632,
    kAliasEpubTextTransform = 633,
    kAliasEpubWordBreak = 634,
    kAliasEpubWritingMode = 635,
    kAliasWebkitAlignContent = 636,
    kAliasWebkitAlignItems = 637,
    kAliasWebkitAlignSelf = 638,
    kAliasWebkitAnimation = 639,
    kAliasWebkitAlternativeAnimationWithTimeline = 640,
    kAliasWebkitAlternativeAnimationWithDelayStartEnd = 641,
    kAliasWebkitAnimationDelay = 642,
    kAliasWebkitAlternativeAnimationDelay = 643,
    kAliasWebkitAnimationDirection = 644,
    kAliasWebkitAnimationDuration = 645,
    kAliasWebkitAnimationFillMode = 646,
    kAliasWebkitAnimationIterationCount = 647,
    kAliasWebkitAnimationName = 648,
    kAliasWebkitAnimationPlayState = 649,
    kAliasWebkitAnimationTimingFunction = 650,
    kAliasWebkitBackfaceVisibility = 651,
    kAliasWebkitBackgroundClip = 652,
    kAliasWebkitBackgroundOrigin = 653,
    kAliasWebkitBackgroundSize = 654,
    kAliasWebkitBorderBottomLeftRadius = 655,
    kAliasWebkitBorderBottomRightRadius = 656,
    kAliasWebkitBorderRadius = 657,
    kAliasWebkitBorderTopLeftRadius = 658,
    kAliasWebkitBorderTopRightRadius = 659,
    kAliasWebkitBoxShadow = 660,
    kAliasWebkitBoxSizing = 661,
    kAliasWebkitClipPath = 662,
    kAliasWebkitColumnCount = 663,
    kAliasWebkitColumnGap = 664,
    kAliasWebkitColumnRule = 665,
    kAliasWebkitColumnRuleColor = 666,
    kAliasWebkitColumnRuleStyle = 667,
    kAliasWebkitColumnRuleWidth = 668,
    kAliasWebkitColumnSpan = 669,
    kAliasWebkitColumnWidth = 670,
    kAliasWebkitColumns = 671,
    kAliasWebkitFilter = 672,
    kAliasWebkitFlex = 673,
    kAliasWebkitFlexBasis = 674,
    kAliasWebkitFlexDirection = 675,
    kAliasWebkitFlexFlow = 676,
    kAliasWebkitFlexGrow = 677,
    kAliasWebkitFlexShrink = 678,
    kAliasWebkitFlexWrap = 679,
    kAliasWebkitFontFeatureSettings = 680,
    kAliasWebkitHyphenateCharacter = 681,
    kAliasWebkitJustifyContent = 682,
    kAliasWebkitOpacity = 683,
    kAliasWebkitOrder = 684,
    kAliasWebkitPerspective = 685,
    kAliasWebkitPerspectiveOrigin = 686,
    kAliasWebkitShapeImageThreshold = 687,
    kAliasWebkitShapeMargin = 688,
    kAliasWebkitShapeOutside = 689,
    kAliasWebkitTextEmphasis = 690,
    kAliasWebkitTextEmphasisColor = 691,
    kAliasWebkitTextEmphasisPosition = 692,
    kAliasWebkitTextEmphasisStyle = 693,
    kAliasWebkitTextSizeAdjust = 694,
    kAliasWebkitTransform = 695,
    kAliasWebkitTransformOrigin = 696,
    kAliasWebkitTransformStyle = 697,
    kAliasWebkitTransition = 698,
    kAliasWebkitTransitionDelay = 699,
    kAliasWebkitTransitionDuration = 700,
    kAliasWebkitTransitionProperty = 701,
    kAliasWebkitTransitionTimingFunction = 702,
    kAliasWebkitUserSelect = 703,
    kAliasWordWrap = 704,
};

const CSSPropertyID kCSSPropertyAliasList[] = {
    CSSPropertyID::kAliasWebkitAppearance,
    CSSPropertyID::kAliasWebkitAppRegion,
    CSSPropertyID::kAliasWebkitAlternativeMaskClip,
    CSSPropertyID::kAliasWebkitAlternativeMaskComposite,
    CSSPropertyID::kAliasWebkitAlternativeMaskImage,
    CSSPropertyID::kAliasWebkitAlternativeMaskOrigin,
    CSSPropertyID::kAliasWebkitAlternativeMaskRepeat,
    CSSPropertyID::kAliasWebkitAlternativeMaskSize,
    CSSPropertyID::kAliasWebkitBorderEndColor,
    CSSPropertyID::kAliasWebkitBorderEndStyle,
    CSSPropertyID::kAliasWebkitBorderEndWidth,
    CSSPropertyID::kAliasWebkitBorderStartColor,
    CSSPropertyID::kAliasWebkitBorderStartStyle,
    CSSPropertyID::kAliasWebkitBorderStartWidth,
    CSSPropertyID::kAliasWebkitBorderBeforeColor,
    CSSPropertyID::kAliasWebkitBorderBeforeStyle,
    CSSPropertyID::kAliasWebkitBorderBeforeWidth,
    CSSPropertyID::kAliasWebkitBorderAfterColor,
    CSSPropertyID::kAliasWebkitBorderAfterStyle,
    CSSPropertyID::kAliasWebkitBorderAfterWidth,
    CSSPropertyID::kAliasWebkitMarginEnd,
    CSSPropertyID::kAliasWebkitMarginStart,
    CSSPropertyID::kAliasWebkitMarginBefore,
    CSSPropertyID::kAliasWebkitMarginAfter,
    CSSPropertyID::kAliasWebkitPaddingEnd,
    CSSPropertyID::kAliasWebkitPaddingStart,
    CSSPropertyID::kAliasWebkitPaddingBefore,
    CSSPropertyID::kAliasWebkitPaddingAfter,
    CSSPropertyID::kAliasWebkitLogicalWidth,
    CSSPropertyID::kAliasWebkitLogicalHeight,
    CSSPropertyID::kAliasWebkitMinLogicalWidth,
    CSSPropertyID::kAliasWebkitMinLogicalHeight,
    CSSPropertyID::kAliasWebkitMaxLogicalWidth,
    CSSPropertyID::kAliasWebkitMaxLogicalHeight,
    CSSPropertyID::kAliasWebkitBorderAfter,
    CSSPropertyID::kAliasWebkitBorderBefore,
    CSSPropertyID::kAliasWebkitBorderEnd,
    CSSPropertyID::kAliasWebkitBorderStart,
    CSSPropertyID::kAliasWebkitAlternativeMask,
    CSSPropertyID::kAliasWebkitAlternativeMaskPosition,
    CSSPropertyID::kAliasEpubCaptionSide,
    CSSPropertyID::kAliasEpubTextCombine,
    CSSPropertyID::kAliasEpubTextEmphasis,
    CSSPropertyID::kAliasEpubTextEmphasisColor,
    CSSPropertyID::kAliasEpubTextEmphasisStyle,
    CSSPropertyID::kAliasEpubTextOrientation,
    CSSPropertyID::kAliasEpubTextTransform,
    CSSPropertyID::kAliasEpubWordBreak,
    CSSPropertyID::kAliasEpubWritingMode,
    CSSPropertyID::kAliasWebkitAlignContent,
    CSSPropertyID::kAliasWebkitAlignItems,
    CSSPropertyID::kAliasWebkitAlignSelf,
    CSSPropertyID::kAliasWebkitAnimation,
    CSSPropertyID::kAliasWebkitAlternativeAnimationWithTimeline,
    CSSPropertyID::kAliasWebkitAlternativeAnimationWithDelayStartEnd,
    CSSPropertyID::kAliasWebkitAnimationDelay,
    CSSPropertyID::kAliasWebkitAlternativeAnimationDelay,
    CSSPropertyID::kAliasWebkitAnimationDirection,
    CSSPropertyID::kAliasWebkitAnimationDuration,
    CSSPropertyID::kAliasWebkitAnimationFillMode,
    CSSPropertyID::kAliasWebkitAnimationIterationCount,
    CSSPropertyID::kAliasWebkitAnimationName,
    CSSPropertyID::kAliasWebkitAnimationPlayState,
    CSSPropertyID::kAliasWebkitAnimationTimingFunction,
    CSSPropertyID::kAliasWebkitBackfaceVisibility,
    CSSPropertyID::kAliasWebkitBackgroundClip,
    CSSPropertyID::kAliasWebkitBackgroundOrigin,
    CSSPropertyID::kAliasWebkitBackgroundSize,
    CSSPropertyID::kAliasWebkitBorderBottomLeftRadius,
    CSSPropertyID::kAliasWebkitBorderBottomRightRadius,
    CSSPropertyID::kAliasWebkitBorderRadius,
    CSSPropertyID::kAliasWebkitBorderTopLeftRadius,
    CSSPropertyID::kAliasWebkitBorderTopRightRadius,
    CSSPropertyID::kAliasWebkitBoxShadow,
    CSSPropertyID::kAliasWebkitBoxSizing,
    CSSPropertyID::kAliasWebkitClipPath,
    CSSPropertyID::kAliasWebkitColumnCount,
    CSSPropertyID::kAliasWebkitColumnGap,
    CSSPropertyID::kAliasWebkitColumnRule,
    CSSPropertyID::kAliasWebkitColumnRuleColor,
    CSSPropertyID::kAliasWebkitColumnRuleStyle,
    CSSPropertyID::kAliasWebkitColumnRuleWidth,
    CSSPropertyID::kAliasWebkitColumnSpan,
    CSSPropertyID::kAliasWebkitColumnWidth,
    CSSPropertyID::kAliasWebkitColumns,
    CSSPropertyID::kAliasWebkitFilter,
    CSSPropertyID::kAliasWebkitFlex,
    CSSPropertyID::kAliasWebkitFlexBasis,
    CSSPropertyID::kAliasWebkitFlexDirection,
    CSSPropertyID::kAliasWebkitFlexFlow,
    CSSPropertyID::kAliasWebkitFlexGrow,
    CSSPropertyID::kAliasWebkitFlexShrink,
    CSSPropertyID::kAliasWebkitFlexWrap,
    CSSPropertyID::kAliasWebkitFontFeatureSettings,
    CSSPropertyID::kAliasWebkitHyphenateCharacter,
    CSSPropertyID::kAliasWebkitJustifyContent,
    CSSPropertyID::kAliasWebkitOpacity,
    CSSPropertyID::kAliasWebkitOrder,
    CSSPropertyID::kAliasWebkitPerspective,
    CSSPropertyID::kAliasWebkitPerspectiveOrigin,
    CSSPropertyID::kAliasWebkitShapeImageThreshold,
    CSSPropertyID::kAliasWebkitShapeMargin,
    CSSPropertyID::kAliasWebkitShapeOutside,
    CSSPropertyID::kAliasWebkitTextEmphasis,
    CSSPropertyID::kAliasWebkitTextEmphasisColor,
    CSSPropertyID::kAliasWebkitTextEmphasisPosition,
    CSSPropertyID::kAliasWebkitTextEmphasisStyle,
    CSSPropertyID::kAliasWebkitTextSizeAdjust,
    CSSPropertyID::kAliasWebkitTransform,
    CSSPropertyID::kAliasWebkitTransformOrigin,
    CSSPropertyID::kAliasWebkitTransformStyle,
    CSSPropertyID::kAliasWebkitTransition,
    CSSPropertyID::kAliasWebkitTransitionDelay,
    CSSPropertyID::kAliasWebkitTransitionDuration,
    CSSPropertyID::kAliasWebkitTransitionProperty,
    CSSPropertyID::kAliasWebkitTransitionTimingFunction,
    CSSPropertyID::kAliasWebkitUserSelect,
    CSSPropertyID::kAliasWordWrap,
};

const CSSPropertyID kCSSComputableProperties[] = {
    CSSPropertyID::kAccentColor,
    CSSPropertyID::kAlignContent,
    CSSPropertyID::kAlignItems,
    CSSPropertyID::kAlignSelf,
    CSSPropertyID::kAlignmentBaseline,
    CSSPropertyID::kAnchorDefault,
    CSSPropertyID::kAnchorName,
    CSSPropertyID::kAnimationComposition,
    CSSPropertyID::kAnimationDelay,
    CSSPropertyID::kAnimationDelayEnd,
    CSSPropertyID::kAnimationDelayStart,
    CSSPropertyID::kAnimationDirection,
    CSSPropertyID::kAnimationDuration,
    CSSPropertyID::kAnimationFillMode,
    CSSPropertyID::kAnimationIterationCount,
    CSSPropertyID::kAnimationName,
    CSSPropertyID::kAnimationPlayState,
    CSSPropertyID::kAnimationRangeEnd,
    CSSPropertyID::kAnimationRangeStart,
    CSSPropertyID::kAnimationTimeline,
    CSSPropertyID::kAnimationTimingFunction,
    CSSPropertyID::kAppRegion,
    CSSPropertyID::kAppearance,
    CSSPropertyID::kBackdropFilter,
    CSSPropertyID::kBackfaceVisibility,
    CSSPropertyID::kBackgroundAttachment,
    CSSPropertyID::kBackgroundBlendMode,
    CSSPropertyID::kBackgroundClip,
    CSSPropertyID::kBackgroundColor,
    CSSPropertyID::kBackgroundImage,
    CSSPropertyID::kBackgroundOrigin,
    CSSPropertyID::kBackgroundPosition,
    CSSPropertyID::kBackgroundRepeat,
    CSSPropertyID::kBackgroundSize,
    CSSPropertyID::kBaselineShift,
    CSSPropertyID::kBaselineSource,
    CSSPropertyID::kBlockSize,
    CSSPropertyID::kBorderBlockEndColor,
    CSSPropertyID::kBorderBlockEndStyle,
    CSSPropertyID::kBorderBlockEndWidth,
    CSSPropertyID::kBorderBlockStartColor,
    CSSPropertyID::kBorderBlockStartStyle,
    CSSPropertyID::kBorderBlockStartWidth,
    CSSPropertyID::kBorderBottomColor,
    CSSPropertyID::kBorderBottomLeftRadius,
    CSSPropertyID::kBorderBottomRightRadius,
    CSSPropertyID::kBorderBottomStyle,
    CSSPropertyID::kBorderBottomWidth,
    CSSPropertyID::kBorderCollapse,
    CSSPropertyID::kBorderEndEndRadius,
    CSSPropertyID::kBorderEndStartRadius,
    CSSPropertyID::kBorderImageOutset,
    CSSPropertyID::kBorderImageRepeat,
    CSSPropertyID::kBorderImageSlice,
    CSSPropertyID::kBorderImageSource,
    CSSPropertyID::kBorderImageWidth,
    CSSPropertyID::kBorderInlineEndColor,
    CSSPropertyID::kBorderInlineEndStyle,
    CSSPropertyID::kBorderInlineEndWidth,
    CSSPropertyID::kBorderInlineStartColor,
    CSSPropertyID::kBorderInlineStartStyle,
    CSSPropertyID::kBorderInlineStartWidth,
    CSSPropertyID::kBorderLeftColor,
    CSSPropertyID::kBorderLeftStyle,
    CSSPropertyID::kBorderLeftWidth,
    CSSPropertyID::kBorderRightColor,
    CSSPropertyID::kBorderRightStyle,
    CSSPropertyID::kBorderRightWidth,
    CSSPropertyID::kBorderStartEndRadius,
    CSSPropertyID::kBorderStartStartRadius,
    CSSPropertyID::kBorderTopColor,
    CSSPropertyID::kBorderTopLeftRadius,
    CSSPropertyID::kBorderTopRightRadius,
    CSSPropertyID::kBorderTopStyle,
    CSSPropertyID::kBorderTopWidth,
    CSSPropertyID::kBottom,
    CSSPropertyID::kBoxShadow,
    CSSPropertyID::kBoxSizing,
    CSSPropertyID::kBreakAfter,
    CSSPropertyID::kBreakBefore,
    CSSPropertyID::kBreakInside,
    CSSPropertyID::kBufferedRendering,
    CSSPropertyID::kCaptionSide,
    CSSPropertyID::kCaretColor,
    CSSPropertyID::kClear,
    CSSPropertyID::kClip,
    CSSPropertyID::kClipPath,
    CSSPropertyID::kClipRule,
    CSSPropertyID::kColor,
    CSSPropertyID::kColorInterpolation,
    CSSPropertyID::kColorInterpolationFilters,
    CSSPropertyID::kColorRendering,
    CSSPropertyID::kColumnCount,
    CSSPropertyID::kColumnGap,
    CSSPropertyID::kColumnRuleColor,
    CSSPropertyID::kColumnRuleStyle,
    CSSPropertyID::kColumnRuleWidth,
    CSSPropertyID::kColumnSpan,
    CSSPropertyID::kColumnWidth,
    CSSPropertyID::kContainIntrinsicBlockSize,
    CSSPropertyID::kContainIntrinsicHeight,
    CSSPropertyID::kContainIntrinsicInlineSize,
    CSSPropertyID::kContainIntrinsicSize,
    CSSPropertyID::kContainIntrinsicWidth,
    CSSPropertyID::kContainerName,
    CSSPropertyID::kContainerType,
    CSSPropertyID::kContent,
    CSSPropertyID::kCursor,
    CSSPropertyID::kCx,
    CSSPropertyID::kCy,
    CSSPropertyID::kD,
    CSSPropertyID::kDirection,
    CSSPropertyID::kDisplay,
    CSSPropertyID::kDominantBaseline,
    CSSPropertyID::kDynamicRangeLimit,
    CSSPropertyID::kEmptyCells,
    CSSPropertyID::kFieldSizing,
    CSSPropertyID::kFill,
    CSSPropertyID::kFillOpacity,
    CSSPropertyID::kFillRule,
    CSSPropertyID::kFilter,
    CSSPropertyID::kFlexBasis,
    CSSPropertyID::kFlexDirection,
    CSSPropertyID::kFlexGrow,
    CSSPropertyID::kFlexShrink,
    CSSPropertyID::kFlexWrap,
    CSSPropertyID::kFloat,
    CSSPropertyID::kFloodColor,
    CSSPropertyID::kFloodOpacity,
    CSSPropertyID::kFontFamily,
    CSSPropertyID::kFontKerning,
    CSSPropertyID::kFontOpticalSizing,
    CSSPropertyID::kFontPalette,
    CSSPropertyID::kFontSize,
    CSSPropertyID::kFontSizeAdjust,
    CSSPropertyID::kFontStretch,
    CSSPropertyID::kFontStyle,
    CSSPropertyID::kFontSynthesisSmallCaps,
    CSSPropertyID::kFontSynthesisStyle,
    CSSPropertyID::kFontSynthesisWeight,
    CSSPropertyID::kFontVariant,
    CSSPropertyID::kFontVariantAlternates,
    CSSPropertyID::kFontVariantCaps,
    CSSPropertyID::kFontVariantEastAsian,
    CSSPropertyID::kFontVariantLigatures,
    CSSPropertyID::kFontVariantNumeric,
    CSSPropertyID::kFontVariantPosition,
    CSSPropertyID::kFontWeight,
    CSSPropertyID::kGridAutoColumns,
    CSSPropertyID::kGridAutoFlow,
    CSSPropertyID::kGridAutoRows,
    CSSPropertyID::kGridColumnEnd,
    CSSPropertyID::kGridColumnStart,
    CSSPropertyID::kGridRowEnd,
    CSSPropertyID::kGridRowStart,
    CSSPropertyID::kGridTemplateAreas,
    CSSPropertyID::kGridTemplateColumns,
    CSSPropertyID::kGridTemplateRows,
    CSSPropertyID::kHeight,
    CSSPropertyID::kHyphenateCharacter,
    CSSPropertyID::kHyphenateLimitChars,
    CSSPropertyID::kHyphens,
    CSSPropertyID::kImageOrientation,
    CSSPropertyID::kImageRendering,
    CSSPropertyID::kInitialLetter,
    CSSPropertyID::kInlineSize,
    CSSPropertyID::kInsetBlockEnd,
    CSSPropertyID::kInsetBlockStart,
    CSSPropertyID::kInsetInlineEnd,
    CSSPropertyID::kInsetInlineStart,
    CSSPropertyID::kIsolation,
    CSSPropertyID::kJustifyContent,
    CSSPropertyID::kJustifyItems,
    CSSPropertyID::kJustifySelf,
    CSSPropertyID::kLeft,
    CSSPropertyID::kLetterSpacing,
    CSSPropertyID::kLightingColor,
    CSSPropertyID::kLineBreak,
    CSSPropertyID::kLineHeight,
    CSSPropertyID::kListStyleImage,
    CSSPropertyID::kListStylePosition,
    CSSPropertyID::kListStyleType,
    CSSPropertyID::kMarginBlockEnd,
    CSSPropertyID::kMarginBlockStart,
    CSSPropertyID::kMarginBottom,
    CSSPropertyID::kMarginInlineEnd,
    CSSPropertyID::kMarginInlineStart,
    CSSPropertyID::kMarginLeft,
    CSSPropertyID::kMarginRight,
    CSSPropertyID::kMarginTop,
    CSSPropertyID::kMarkerEnd,
    CSSPropertyID::kMarkerMid,
    CSSPropertyID::kMarkerStart,
    CSSPropertyID::kMaskClip,
    CSSPropertyID::kMaskComposite,
    CSSPropertyID::kMaskImage,
    CSSPropertyID::kMaskMode,
    CSSPropertyID::kMaskOrigin,
    CSSPropertyID::kMaskPosition,
    CSSPropertyID::kMaskRepeat,
    CSSPropertyID::kMaskSize,
    CSSPropertyID::kMaskType,
    CSSPropertyID::kMathDepth,
    CSSPropertyID::kMathShift,
    CSSPropertyID::kMathStyle,
    CSSPropertyID::kMaxBlockSize,
    CSSPropertyID::kMaxHeight,
    CSSPropertyID::kMaxInlineSize,
    CSSPropertyID::kMaxWidth,
    CSSPropertyID::kMinBlockSize,
    CSSPropertyID::kMinHeight,
    CSSPropertyID::kMinInlineSize,
    CSSPropertyID::kMinWidth,
    CSSPropertyID::kMixBlendMode,
    CSSPropertyID::kObjectFit,
    CSSPropertyID::kObjectPosition,
    CSSPropertyID::kObjectViewBox,
    CSSPropertyID::kOffsetAnchor,
    CSSPropertyID::kOffsetDistance,
    CSSPropertyID::kOffsetPath,
    CSSPropertyID::kOffsetPosition,
    CSSPropertyID::kOffsetRotate,
    CSSPropertyID::kOpacity,
    CSSPropertyID::kOrder,
    CSSPropertyID::kOrphans,
    CSSPropertyID::kOutlineColor,
    CSSPropertyID::kOutlineOffset,
    CSSPropertyID::kOutlineStyle,
    CSSPropertyID::kOutlineWidth,
    CSSPropertyID::kOverflowAnchor,
    CSSPropertyID::kOverflowBlock,
    CSSPropertyID::kOverflowClipMargin,
    CSSPropertyID::kOverflowInline,
    CSSPropertyID::kOverflowWrap,
    CSSPropertyID::kOverflowX,
    CSSPropertyID::kOverflowY,
    CSSPropertyID::kOverlay,
    CSSPropertyID::kOverscrollBehaviorBlock,
    CSSPropertyID::kOverscrollBehaviorInline,
    CSSPropertyID::kPaddingBlockEnd,
    CSSPropertyID::kPaddingBlockStart,
    CSSPropertyID::kPaddingBottom,
    CSSPropertyID::kPaddingInlineEnd,
    CSSPropertyID::kPaddingInlineStart,
    CSSPropertyID::kPaddingLeft,
    CSSPropertyID::kPaddingRight,
    CSSPropertyID::kPaddingTop,
    CSSPropertyID::kPaintOrder,
    CSSPropertyID::kPerspective,
    CSSPropertyID::kPerspectiveOrigin,
    CSSPropertyID::kPointerEvents,
    CSSPropertyID::kPopoverHideDelay,
    CSSPropertyID::kPopoverShowDelay,
    CSSPropertyID::kPosition,
    CSSPropertyID::kPositionFallback,
    CSSPropertyID::kPositionFallbackBounds,
    CSSPropertyID::kR,
    CSSPropertyID::kResize,
    CSSPropertyID::kRight,
    CSSPropertyID::kRotate,
    CSSPropertyID::kRowGap,
    CSSPropertyID::kRubyPosition,
    CSSPropertyID::kRx,
    CSSPropertyID::kRy,
    CSSPropertyID::kScale,
    CSSPropertyID::kScrollBehavior,
    CSSPropertyID::kScrollMarginBlockEnd,
    CSSPropertyID::kScrollMarginBlockStart,
    CSSPropertyID::kScrollMarginInlineEnd,
    CSSPropertyID::kScrollMarginInlineStart,
    CSSPropertyID::kScrollPaddingBlockEnd,
    CSSPropertyID::kScrollPaddingBlockStart,
    CSSPropertyID::kScrollPaddingInlineEnd,
    CSSPropertyID::kScrollPaddingInlineStart,
    CSSPropertyID::kScrollStartBlock,
    CSSPropertyID::kScrollStartInline,
    CSSPropertyID::kScrollStartTargetBlock,
    CSSPropertyID::kScrollStartTargetInline,
    CSSPropertyID::kScrollStartTargetX,
    CSSPropertyID::kScrollStartTargetY,
    CSSPropertyID::kScrollStartX,
    CSSPropertyID::kScrollStartY,
    CSSPropertyID::kScrollTimelineAxis,
    CSSPropertyID::kScrollTimelineName,
    CSSPropertyID::kScrollbarColor,
    CSSPropertyID::kScrollbarGutter,
    CSSPropertyID::kScrollbarWidth,
    CSSPropertyID::kShapeImageThreshold,
    CSSPropertyID::kShapeMargin,
    CSSPropertyID::kShapeOutside,
    CSSPropertyID::kShapeRendering,
    CSSPropertyID::kSpeak,
    CSSPropertyID::kStopColor,
    CSSPropertyID::kStopOpacity,
    CSSPropertyID::kStroke,
    CSSPropertyID::kStrokeDasharray,
    CSSPropertyID::kStrokeDashoffset,
    CSSPropertyID::kStrokeLinecap,
    CSSPropertyID::kStrokeLinejoin,
    CSSPropertyID::kStrokeMiterlimit,
    CSSPropertyID::kStrokeOpacity,
    CSSPropertyID::kStrokeWidth,
    CSSPropertyID::kTabSize,
    CSSPropertyID::kTableLayout,
    CSSPropertyID::kTextAlign,
    CSSPropertyID::kTextAlignLast,
    CSSPropertyID::kTextAnchor,
    CSSPropertyID::kTextAutospace,
    CSSPropertyID::kTextBoxTrim,
    CSSPropertyID::kTextDecoration,
    CSSPropertyID::kTextDecorationColor,
    CSSPropertyID::kTextDecorationLine,
    CSSPropertyID::kTextDecorationSkipInk,
    CSSPropertyID::kTextDecorationStyle,
    CSSPropertyID::kTextEmphasisColor,
    CSSPropertyID::kTextEmphasisPosition,
    CSSPropertyID::kTextEmphasisStyle,
    CSSPropertyID::kTextIndent,
    CSSPropertyID::kTextOverflow,
    CSSPropertyID::kTextRendering,
    CSSPropertyID::kTextShadow,
    CSSPropertyID::kTextSizeAdjust,
    CSSPropertyID::kTextSpacingTrim,
    CSSPropertyID::kTextTransform,
    CSSPropertyID::kTextUnderlinePosition,
    CSSPropertyID::kTextWrap,
    CSSPropertyID::kTimelineScope,
    CSSPropertyID::kTop,
    CSSPropertyID::kTouchAction,
    CSSPropertyID::kTransform,
    CSSPropertyID::kTransformOrigin,
    CSSPropertyID::kTransformStyle,
    CSSPropertyID::kTransitionBehavior,
    CSSPropertyID::kTransitionDelay,
    CSSPropertyID::kTransitionDuration,
    CSSPropertyID::kTransitionProperty,
    CSSPropertyID::kTransitionTimingFunction,
    CSSPropertyID::kTranslate,
    CSSPropertyID::kUnicodeBidi,
    CSSPropertyID::kUserSelect,
    CSSPropertyID::kVectorEffect,
    CSSPropertyID::kVerticalAlign,
    CSSPropertyID::kViewTimelineAxis,
    CSSPropertyID::kViewTimelineInset,
    CSSPropertyID::kViewTimelineName,
    CSSPropertyID::kViewTransitionName,
    CSSPropertyID::kVisibility,
    CSSPropertyID::kWhiteSpaceCollapse,
    CSSPropertyID::kWidows,
    CSSPropertyID::kWidth,
    CSSPropertyID::kWillChange,
    CSSPropertyID::kWordBreak,
    CSSPropertyID::kWordSpacing,
    CSSPropertyID::kWritingMode,
    CSSPropertyID::kX,
    CSSPropertyID::kY,
    CSSPropertyID::kZIndex,
    CSSPropertyID::kZoom,
    CSSPropertyID::kWebkitBorderHorizontalSpacing,
    CSSPropertyID::kWebkitBorderImage,
    CSSPropertyID::kWebkitBorderVerticalSpacing,
    CSSPropertyID::kWebkitBoxAlign,
    CSSPropertyID::kWebkitBoxDecorationBreak,
    CSSPropertyID::kWebkitBoxDirection,
    CSSPropertyID::kWebkitBoxDirectionAlternative,
    CSSPropertyID::kWebkitBoxFlex,
    CSSPropertyID::kWebkitBoxOrdinalGroup,
    CSSPropertyID::kWebkitBoxOrient,
    CSSPropertyID::kWebkitBoxPack,
    CSSPropertyID::kWebkitBoxReflect,
    CSSPropertyID::kWebkitFontSmoothing,
    CSSPropertyID::kWebkitLineBreak,
    CSSPropertyID::kWebkitLineClamp,
    CSSPropertyID::kWebkitLocale,
    CSSPropertyID::kWebkitMaskBoxImage,
    CSSPropertyID::kWebkitMaskBoxImageOutset,
    CSSPropertyID::kWebkitMaskBoxImageRepeat,
    CSSPropertyID::kWebkitMaskBoxImageSlice,
    CSSPropertyID::kWebkitMaskBoxImageSource,
    CSSPropertyID::kWebkitMaskBoxImageWidth,
    CSSPropertyID::kWebkitMaskClip,
    CSSPropertyID::kWebkitMaskComposite,
    CSSPropertyID::kWebkitMaskImage,
    CSSPropertyID::kWebkitMaskOrigin,
    CSSPropertyID::kWebkitMaskPosition,
    CSSPropertyID::kWebkitMaskRepeat,
    CSSPropertyID::kWebkitMaskSize,
    CSSPropertyID::kWebkitPrintColorAdjust,
    CSSPropertyID::kWebkitRtlOrdering,
    CSSPropertyID::kWebkitTapHighlightColor,
    CSSPropertyID::kWebkitTextCombine,
    CSSPropertyID::kWebkitTextDecorationsInEffect,
    CSSPropertyID::kWebkitTextFillColor,
    CSSPropertyID::kWebkitTextOrientation,
    CSSPropertyID::kWebkitTextSecurity,
    CSSPropertyID::kWebkitTextStrokeColor,
    CSSPropertyID::kWebkitTextStrokeWidth,
    CSSPropertyID::kWebkitUserDrag,
    CSSPropertyID::kWebkitUserModify,
    CSSPropertyID::kWebkitWritingMode,
};

// The lowest CSSPropertyID excluding kInvalid and kVariable.
const int kIntFirstCSSProperty = 2;
const CSSPropertyID kFirstCSSProperty =
    static_cast<CSSPropertyID>(kIntFirstCSSProperty);

// The number of unresolved CSS properties excluding kInvalid and kVariable.
const int kNumCSSProperties = 703;

// The highest resolved CSSPropertyID.
const int kIntLastCSSProperty = 586;
const CSSPropertyID kLastCSSProperty =
    static_cast<CSSPropertyID>(kIntLastCSSProperty);

// The highest unresolved CSSPropertyID.
const CSSPropertyID kLastUnresolvedCSSProperty =
    static_cast<CSSPropertyID>(704);

const CSSPropertyID kFirstHighPriorityCSSProperty = kFirstCSSProperty;
const CSSPropertyID kLastHighPriorityCSSProperty = CSSPropertyID::kZoom;

// 1 + <The highest unresolved CSSPropertyID>.
const int kNumCSSPropertyIDs = static_cast<int>(kLastUnresolvedCSSProperty) + 1;

const size_t kMaxCSSPropertyNameLength = 48;
constexpr size_t kCSSPropertyIDBitLength = 10;
constexpr size_t kMaxShorthandExpansion = 17;

static_assert((static_cast<size_t>(1) << kCSSPropertyIDBitLength) >
                  static_cast<size_t>(kLastUnresolvedCSSProperty),
              "kCSSPropertyIDBitLength has enough bits");

static_assert(CSSPropertyID::kColor == kFirstHighPriorityCSSProperty,
              "CSSPropertyID::kColor is the first high-priority property");
static_assert(CSSPropertyID::kZoom == kLastHighPriorityCSSProperty,
              "CSSPropertyID::kZoom is the last high-priority property");
static_assert((static_cast<int>(kLastHighPriorityCSSProperty) -
               static_cast<int>(kFirstHighPriorityCSSProperty)) == 32,
              "There should a low number of high-priority properties");

inline int GetCSSPropertyIDIndex(CSSPropertyID id) {
    DCHECK_GE(id, kFirstCSSProperty);
    DCHECK_LE(id, kLastCSSProperty);
    return static_cast<int>(id) - kIntFirstCSSProperty;
}

constexpr bool IsHighPriority(CSSPropertyID id) {
  return id >= kFirstHighPriorityCSSProperty &&
      id <= kLastHighPriorityCSSProperty;
}

inline bool IsCSSPropertyIDWithName(CSSPropertyID id)
{
    return id >= kFirstCSSProperty && id <= kLastUnresolvedCSSProperty;
}

inline bool IsValidCSSPropertyID(CSSPropertyID id)
{
    return id != CSSPropertyID::kInvalid;
}

inline CSSPropertyID ConvertToCSSPropertyID(int value)
{
    DCHECK_GE(value, static_cast<int>(CSSPropertyID::kInvalid));
    DCHECK_LE(value, kIntLastCSSProperty);
    return static_cast<CSSPropertyID>(value);
}

int CORE_EXPORT ResolveCSSPropertyAlias(int value);

inline bool IsPropertyAlias(CSSPropertyID id) {
  return static_cast<int>(id) >= 587;
}

inline CSSPropertyID ResolveCSSPropertyID(CSSPropertyID id)
{
  int int_id = static_cast<int>(id);
  if (IsPropertyAlias(id))
    int_id = ResolveCSSPropertyAlias(int_id);
  return ConvertToCSSPropertyID(int_id);
}

CSSPropertyID CORE_EXPORT CssPropertyID(const ExecutionContext*,
                                        const WTF::String&);

class CSSPropertyIDList {
  STACK_ALLOCATED();

 public:
  class Iterator {
    STACK_ALLOCATED();
   public:
    Iterator(int id) : id_(id) {}
    CSSPropertyID operator*() const { return ConvertToCSSPropertyID(id_); }
    Iterator& operator++() {
      id_++;
      return *this;
    }
    bool operator!=(const Iterator& i) const { return id_ != i.id_; }

   private:
    int id_;
  };
  Iterator begin() const { return Iterator(kIntFirstCSSProperty); }
  Iterator end() const { return Iterator(kIntLastCSSProperty + 1); }
};

mojom::blink::CSSSampleId CORE_EXPORT GetCSSSampleId(CSSPropertyID id);

}  // namespace blink

// Even though these hash traits are not affected by code generation,
// it's important to define them in the same file as CSSPropertyID to avoid
// hard-to-diagnose problems from neglecting to include said traits.
// See https://crbug.com/1246970.
namespace WTF {
template <>
struct HashTraits<blink::CSSPropertyID>
    : EnumHashTraits<blink::CSSPropertyID,
                     blink::CSSPropertyID::kInvalid,
                     blink::kNumCSSPropertyIDs> {};
}  // namespace WTF

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_
