// Copyright 2014 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_

#include <string.h>
#include <stdint.h>

#include "third_party/blink/renderer/core/core_export.h"
#include "third_party/blink/renderer/core/css/parser/css_parser_mode.h"

namespace blink {

enum class CSSValueID {
  kInvalid = 0,
  kInherit = 1,
  kInitial = 2,
  kNone = 3,
  kHidden = 4,
  kInset = 5,
  kGroove = 6,
  kOutset = 7,
  kRidge = 8,
  kDotted = 9,
  kDashed = 10,
  kSolid = 11,
  kDouble = 12,
  kCaption = 13,
  kIcon = 14,
  kMenu = 15,
  kMessageBox = 16,
  kSmallCaption = 17,
  kWebkitMiniControl = 18,
  kWebkitSmallControl = 19,
  kWebkitControl = 20,
  kStatusBar = 21,
  kItalic = 22,
  kOblique = 23,
  kAll = 24,
  kCommonLigatures = 25,
  kNoCommonLigatures = 26,
  kDiscretionaryLigatures = 27,
  kNoDiscretionaryLigatures = 28,
  kHistoricalLigatures = 29,
  kNoHistoricalLigatures = 30,
  kContextual = 31,
  kNoContextual = 32,
  kSmallCaps = 33,
  kAllSmallCaps = 34,
  kPetiteCaps = 35,
  kAllPetiteCaps = 36,
  kUnicase = 37,
  kTitlingCaps = 38,
  kJis78 = 39,
  kJis83 = 40,
  kJis90 = 41,
  kJis04 = 42,
  kSimplified = 43,
  kTraditional = 44,
  kFullWidth = 45,
  kProportionalWidth = 46,
  kRuby = 47,
  kLiningNums = 48,
  kOldstyleNums = 49,
  kProportionalNums = 50,
  kTabularNums = 51,
  kDiagonalFractions = 52,
  kStackedFractions = 53,
  kOrdinal = 54,
  kSlashedZero = 55,
  kStylistic = 56,
  kHistoricalForms = 57,
  kStyleset = 58,
  kCharacterVariant = 59,
  kSwash = 60,
  kOrnaments = 61,
  kAnnotation = 62,
  kNormal = 63,
  kBold = 64,
  kBolder = 65,
  kLighter = 66,
  kWeight = 67,
  kUltraCondensed = 68,
  kExtraCondensed = 69,
  kCondensed = 70,
  kSemiCondensed = 71,
  kSemiExpanded = 72,
  kExpanded = 73,
  kExtraExpanded = 74,
  kUltraExpanded = 75,
  kXxSmall = 76,
  kXSmall = 77,
  kSmall = 78,
  kMedium = 79,
  kLarge = 80,
  kXLarge = 81,
  kXxLarge = 82,
  kXxxLarge = 83,
  kSmaller = 84,
  kLarger = 85,
  kWebkitXxxLarge = 86,
  kExHeight = 87,
  kCapHeight = 88,
  kChWidth = 89,
  kIcWidth = 90,
  kSerif = 91,
  kSansSerif = 92,
  kCursive = 93,
  kFantasy = 94,
  kMonospace = 95,
  kSystemUi = 96,
  kWebkitBody = 97,
  kMath = 98,
  kSwap = 99,
  kFallback = 100,
  kOptional = 101,
  kFontTech = 102,
  kFontFormat = 103,
  kPaletteMix = 104,
  kAqua = 105,
  kBlack = 106,
  kBlue = 107,
  kFuchsia = 108,
  kGray = 109,
  kGreen = 110,
  kLime = 111,
  kMaroon = 112,
  kNavy = 113,
  kOlive = 114,
  kOrange = 115,
  kPurple = 116,
  kRed = 117,
  kSilver = 118,
  kTeal = 119,
  kWhite = 120,
  kYellow = 121,
  kTransparent = 122,
  kWebkitLink = 123,
  kWebkitActivelink = 124,
  kAccentcolor = 125,
  kAccentcolortext = 126,
  kActiveborder = 127,
  kActivecaption = 128,
  kActivetext = 129,
  kAppworkspace = 130,
  kBackground = 131,
  kButtonborder = 132,
  kButtonface = 133,
  kButtonhighlight = 134,
  kButtonshadow = 135,
  kButtontext = 136,
  kCanvas = 137,
  kCanvastext = 138,
  kCaptiontext = 139,
  kField = 140,
  kFieldtext = 141,
  kGraytext = 142,
  kHighlight = 143,
  kHighlighttext = 144,
  kInactiveborder = 145,
  kInactivecaption = 146,
  kInactivecaptiontext = 147,
  kInfobackground = 148,
  kInfotext = 149,
  kLinktext = 150,
  kMark = 151,
  kMarktext = 152,
  kMenutext = 153,
  kSelecteditem = 154,
  kSelecteditemtext = 155,
  kScrollbar = 156,
  kThreeddarkshadow = 157,
  kThreedface = 158,
  kThreedhighlight = 159,
  kThreedlightshadow = 160,
  kThreedshadow = 161,
  kVisitedtext = 162,
  kWindow = 163,
  kWindowframe = 164,
  kWindowtext = 165,
  kInternalActiveListBoxSelection = 166,
  kInternalActiveListBoxSelectionText = 167,
  kInternalInactiveListBoxSelection = 168,
  kInternalInactiveListBoxSelectionText = 169,
  kWebkitFocusRingColor = 170,
  kCurrentcolor = 171,
  kGrey = 172,
  kInternalQuirkInherit = 173,
  kInternalSpellingErrorColor = 174,
  kInternalGrammarErrorColor = 175,
  kAliceblue = 176,
  kAntiquewhite = 177,
  kAquamarine = 178,
  kAzure = 179,
  kBeige = 180,
  kBisque = 181,
  kBlanchedalmond = 182,
  kBlueviolet = 183,
  kBrown = 184,
  kBurlywood = 185,
  kCadetblue = 186,
  kChartreuse = 187,
  kChocolate = 188,
  kCoral = 189,
  kCornflowerblue = 190,
  kCornsilk = 191,
  kCrimson = 192,
  kCyan = 193,
  kDarkblue = 194,
  kDarkcyan = 195,
  kDarkgoldenrod = 196,
  kDarkgray = 197,
  kDarkgreen = 198,
  kDarkgrey = 199,
  kDarkkhaki = 200,
  kDarkmagenta = 201,
  kDarkolivegreen = 202,
  kDarkorange = 203,
  kDarkorchid = 204,
  kDarkred = 205,
  kDarksalmon = 206,
  kDarkseagreen = 207,
  kDarkslateblue = 208,
  kDarkslategray = 209,
  kDarkslategrey = 210,
  kDarkturquoise = 211,
  kDarkviolet = 212,
  kDeeppink = 213,
  kDeepskyblue = 214,
  kDimgray = 215,
  kDimgrey = 216,
  kDodgerblue = 217,
  kFirebrick = 218,
  kFloralwhite = 219,
  kForestgreen = 220,
  kGainsboro = 221,
  kGhostwhite = 222,
  kGold = 223,
  kGoldenrod = 224,
  kGreenyellow = 225,
  kHoneydew = 226,
  kHotpink = 227,
  kIndianred = 228,
  kIndigo = 229,
  kIvory = 230,
  kKhaki = 231,
  kLavender = 232,
  kLavenderblush = 233,
  kLawngreen = 234,
  kLemonchiffon = 235,
  kLightblue = 236,
  kLightcoral = 237,
  kLightcyan = 238,
  kLightgoldenrodyellow = 239,
  kLightgray = 240,
  kLightgreen = 241,
  kLightgrey = 242,
  kLightpink = 243,
  kLightsalmon = 244,
  kLightseagreen = 245,
  kLightskyblue = 246,
  kLightslategray = 247,
  kLightslategrey = 248,
  kLightsteelblue = 249,
  kLightyellow = 250,
  kLimegreen = 251,
  kLinen = 252,
  kMagenta = 253,
  kMediumaquamarine = 254,
  kMediumblue = 255,
  kMediumorchid = 256,
  kMediumpurple = 257,
  kMediumseagreen = 258,
  kMediumslateblue = 259,
  kMediumspringgreen = 260,
  kMediumturquoise = 261,
  kMediumvioletred = 262,
  kMidnightblue = 263,
  kMintcream = 264,
  kMistyrose = 265,
  kMoccasin = 266,
  kNavajowhite = 267,
  kOldlace = 268,
  kOlivedrab = 269,
  kOrangered = 270,
  kOrchid = 271,
  kPalegoldenrod = 272,
  kPalegreen = 273,
  kPaleturquoise = 274,
  kPalevioletred = 275,
  kPapayawhip = 276,
  kPeachpuff = 277,
  kPeru = 278,
  kPink = 279,
  kPlum = 280,
  kPowderblue = 281,
  kRebeccapurple = 282,
  kRosybrown = 283,
  kRoyalblue = 284,
  kSaddlebrown = 285,
  kSalmon = 286,
  kSandybrown = 287,
  kSeagreen = 288,
  kSeashell = 289,
  kSienna = 290,
  kSkyblue = 291,
  kSlateblue = 292,
  kSlategray = 293,
  kSlategrey = 294,
  kSnow = 295,
  kSpringgreen = 296,
  kSteelblue = 297,
  kTan = 298,
  kThistle = 299,
  kTomato = 300,
  kTurquoise = 301,
  kViolet = 302,
  kWheat = 303,
  kWhitesmoke = 304,
  kYellowgreen = 305,
  kRepeat = 306,
  kRepeatX = 307,
  kRepeatY = 308,
  kNoRepeat = 309,
  kClear = 310,
  kCopy = 311,
  kSourceOver = 312,
  kSourceIn = 313,
  kSourceOut = 314,
  kSourceAtop = 315,
  kDestinationOver = 316,
  kDestinationIn = 317,
  kDestinationOut = 318,
  kDestinationAtop = 319,
  kXor = 320,
  kPlusLighter = 321,
  kSubtract = 322,
  kIntersect = 323,
  kExclude = 324,
  kBaseline = 325,
  kMiddle = 326,
  kSub = 327,
  kSuper = 328,
  kTextTop = 329,
  kTextBottom = 330,
  kTop = 331,
  kBottom = 332,
  kWebkitBaselineMiddle = 333,
  kWebkitAuto = 334,
  kLeft = 335,
  kRight = 336,
  kCenter = 337,
  kJustify = 338,
  kWebkitLeft = 339,
  kWebkitRight = 340,
  kWebkitCenter = 341,
  kWebkitMatchParent = 342,
  kInternalCenter = 343,
  kInlineStart = 344,
  kInlineEnd = 345,
  kOutside = 346,
  kInside = 347,
  kDisc = 348,
  kCircle = 349,
  kSquare = 350,
  kDisclosureOpen = 351,
  kDisclosureClosed = 352,
  kDecimal = 353,
  kInline = 354,
  kBlock = 355,
  kFlowRoot = 356,
  kFlow = 357,
  kTable = 358,
  kFlex = 359,
  kGrid = 360,
  kContents = 361,
  kTableRowGroup = 362,
  kTableHeaderGroup = 363,
  kTableFooterGroup = 364,
  kTableRow = 365,
  kTableColumnGroup = 366,
  kTableColumn = 367,
  kTableCell = 368,
  kTableCaption = 369,
  kInlineBlock = 370,
  kInlineTable = 371,
  kInlineFlex = 372,
  kInlineGrid = 373,
  kWebkitBox = 374,
  kWebkitInlineBox = 375,
  kWebkitFlex = 376,
  kWebkitInlineFlex = 377,
  kLayout = 378,
  kInlineLayout = 379,
  kListItem = 380,
  kAuto = 381,
  kCrosshair = 382,
  kDefault = 383,
  kPointer = 384,
  kMove = 385,
  kVerticalText = 386,
  kCell = 387,
  kContextMenu = 388,
  kAlias = 389,
  kProgress = 390,
  kNoDrop = 391,
  kNotAllowed = 392,
  kZoomIn = 393,
  kZoomOut = 394,
  kEResize = 395,
  kNeResize = 396,
  kNwResize = 397,
  kNResize = 398,
  kSeResize = 399,
  kSwResize = 400,
  kSResize = 401,
  kWResize = 402,
  kEwResize = 403,
  kNsResize = 404,
  kNeswResize = 405,
  kNwseResize = 406,
  kColResize = 407,
  kRowResize = 408,
  kText = 409,
  kWait = 410,
  kHelp = 411,
  kAllScroll = 412,
  kGrab = 413,
  kGrabbing = 414,
  kWebkitGrab = 415,
  kWebkitGrabbing = 416,
  kWebkitZoomIn = 417,
  kWebkitZoomOut = 418,
  kLtr = 419,
  kRtl = 420,
  kCapitalize = 421,
  kUppercase = 422,
  kLowercase = 423,
  kMathAuto = 424,
  kVisible = 425,
  kCollapse = 426,
  kPreserve = 427,
  kPreserveBreaks = 428,
  kPretty = 429,
  kA3 = 430,
  kA4 = 431,
  kA5 = 432,
  kAbove = 433,
  kAbsolute = 434,
  kAlways = 435,
  kAvoid = 436,
  kB4 = 437,
  kB5 = 438,
  kBelow = 439,
  kBidiOverride = 440,
  kBlink = 441,
  kBoth = 442,
  kBreakSpaces = 443,
  kCloseQuote = 444,
  kEmbed = 445,
  kFixed = 446,
  kHand = 447,
  kHide = 448,
  kIsolate = 449,
  kIsolateOverride = 450,
  kPlaintext = 451,
  kWebkitIsolate = 452,
  kWebkitIsolateOverride = 453,
  kWebkitPlaintext = 454,
  kJisB5 = 455,
  kJisB4 = 456,
  kLandscape = 457,
  kLedger = 458,
  kLegal = 459,
  kLetter = 460,
  kLineThrough = 461,
  kLocal = 462,
  kNoCloseQuote = 463,
  kNoOpenQuote = 464,
  kNowrap = 465,
  kOpenQuote = 466,
  kOverlay = 467,
  kOverline = 468,
  kPortrait = 469,
  kPre = 470,
  kPreLine = 471,
  kPreWrap = 472,
  kRelative = 473,
  kScroll = 474,
  kSeparate = 475,
  kShow = 476,
  kStatic = 477,
  kThick = 478,
  kThin = 479,
  kUnderline = 480,
  kView = 481,
  kWavy = 482,
  kCompact = 483,
  kStretch = 484,
  kStart = 485,
  kEnd = 486,
  kClone = 487,
  kSlice = 488,
  kReverse = 489,
  kHorizontal = 490,
  kVertical = 491,
  kInlineAxis = 492,
  kBlockAxis = 493,
  kFlexStart = 494,
  kFlexEnd = 495,
  kSpaceBetween = 496,
  kSpaceAround = 497,
  kSpaceEvenly = 498,
  kUnsafe = 499,
  kSafe = 500,
  kAnchorCenter = 501,
  kRow = 502,
  kRowReverse = 503,
  kColumn = 504,
  kColumnReverse = 505,
  kWrap = 506,
  kWrapReverse = 507,
  kAutoFlow = 508,
  kDense = 509,
  kReadOnly = 510,
  kReadWrite = 511,
  kReadWritePlaintextOnly = 512,
  kElement = 513,
  kWebkitMinContent = 514,
  kWebkitMaxContent = 515,
  kWebkitFillAvailable = 516,
  kWebkitFitContent = 517,
  kMinContent = 518,
  kMaxContent = 519,
  kFitContent = 520,
  kNoAutospace = 521,
  kClip = 522,
  kEllipsis = 523,
  kSpellingError = 524,
  kGrammarError = 525,
  kFromFont = 526,
  kSpaceAll = 527,
  kSpaceFirst = 528,
  kBreakAll = 529,
  kKeepAll = 530,
  kAutoPhrase = 531,
  kBreakWord = 532,
  kSpace = 533,
  kLoose = 534,
  kStrict = 535,
  kAfterWhiteSpace = 536,
  kAnywhere = 537,
  kManual = 538,
  kCheckbox = 539,
  kRadio = 540,
  kButton = 541,
  kListbox = 542,
  kInternalMediaControl = 543,
  kMenulist = 544,
  kMenulistButton = 545,
  kMeter = 546,
  kProgressBar = 547,
  kSearchfield = 548,
  kTextfield = 549,
  kTextarea = 550,
  kInnerSpinButton = 551,
  kPushButton = 552,
  kSquareButton = 553,
  kSliderHorizontal = 554,
  kSearchfieldCancelButton = 555,
  kMediaSlider = 556,
  kMediaSliderthumb = 557,
  kMediaVolumeSlider = 558,
  kMediaVolumeSliderthumb = 559,
  kSliderthumbHorizontal = 560,
  kSliderthumbVertical = 561,
  kSliderVertical = 562,
  kRound = 563,
  kBorder = 564,
  kBorderBox = 565,
  kContent = 566,
  kContentBox = 567,
  kPadding = 568,
  kPaddingBox = 569,
  kMarginBox = 570,
  kNoClip = 571,
  kContain = 572,
  kCover = 573,
  kLogical = 574,
  kVisual = 575,
  kReplace = 576,
  kAccumulate = 577,
  kAlternate = 578,
  kAlternateReverse = 579,
  kForwards = 580,
  kBackwards = 581,
  kInfinite = 582,
  kRunning = 583,
  kPaused = 584,
  kFlat = 585,
  kPreserve3d = 586,
  kFillBox = 587,
  kViewBox = 588,
  kEase = 589,
  kLinear = 590,
  kEaseIn = 591,
  kEaseOut = 592,
  kEaseInOut = 593,
  kJumpBoth = 594,
  kJumpEnd = 595,
  kJumpNone = 596,
  kJumpStart = 597,
  kStepStart = 598,
  kStepEnd = 599,
  kSteps = 600,
  kFrames = 601,
  kCubicBezier = 602,
  kDocument = 603,
  kReset = 604,
  kZoom = 605,
  kVisiblepainted = 606,
  kVisiblefill = 607,
  kVisiblestroke = 608,
  kPainted = 609,
  kFill = 610,
  kStroke = 611,
  kBoundingBox = 612,
  kSpellOut = 613,
  kDigits = 614,
  kLiteralPunctuation = 615,
  kNoPunctuation = 616,
  kAntialiased = 617,
  kSubpixelAntialiased = 618,
  kOptimizespeed = 619,
  kOptimizelegibility = 620,
  kGeometricprecision = 621,
  kCrispedges = 622,
  kEconomy = 623,
  kExact = 624,
  kLr = 625,
  kRl = 626,
  kTb = 627,
  kLrTb = 628,
  kRlTb = 629,
  kTbRl = 630,
  kHorizontalTb = 631,
  kVerticalRl = 632,
  kVerticalLr = 633,
  kAfter = 634,
  kBefore = 635,
  kOver = 636,
  kUnder = 637,
  kFilled = 638,
  kOpen = 639,
  kDot = 640,
  kDoubleCircle = 641,
  kTriangle = 642,
  kSesame = 643,
  kEllipse = 644,
  kClosestSide = 645,
  kClosestCorner = 646,
  kFarthestSide = 647,
  kFarthestCorner = 648,
  kMixed = 649,
  kSideways = 650,
  kSidewaysRight = 651,
  kUpright = 652,
  kVerticalRight = 653,
  kOn = 654,
  kOff = 655,
  kOptimizequality = 656,
  kPixelated = 657,
  kWebkitOptimizeContrast = 658,
  kFromImage = 659,
  kRotateLeft = 660,
  kRotateRight = 661,
  kNonzero = 662,
  kEvenodd = 663,
  kAt = 664,
  kAlphabetic = 665,
  kBorderless = 666,
  kFullscreen = 667,
  kStandalone = 668,
  kMinimalUi = 669,
  kBrowser = 670,
  kWindowControlsOverlay = 671,
  kTabbed = 672,
  kMinimized = 673,
  kMaximized = 674,
  kPaged = 675,
  kSlow = 676,
  kFast = 677,
  kSticky = 678,
  kCoarse = 679,
  kFine = 680,
  kOnDemand = 681,
  kHover = 682,
  kMultiply = 683,
  kScreen = 684,
  kDarken = 685,
  kLighten = 686,
  kColorDodge = 687,
  kColorBurn = 688,
  kHardLight = 689,
  kSoftLight = 690,
  kDifference = 691,
  kExclusion = 692,
  kHue = 693,
  kSaturation = 694,
  kColor = 695,
  kLuminosity = 696,
  kScaleDown = 697,
  kBalance = 698,
  kDrag = 699,
  kNoDrag = 700,
  kSpan = 701,
  kMinmax = 702,
  kSubgrid = 703,
  kProgressive = 704,
  kInterlace = 705,
  kMarkers = 706,
  kAlpha = 707,
  kLuminance = 708,
  kMatchSource = 709,
  kSRGB = 710,
  kLinearrgb = 711,
  kButt = 712,
  kMiter = 713,
  kBevel = 714,
  kBeforeEdge = 715,
  kAfterEdge = 716,
  kCentral = 717,
  kTextBeforeEdge = 718,
  kTextAfterEdge = 719,
  kIdeographic = 720,
  kHanging = 721,
  kMathematical = 722,
  kUseScript = 723,
  kNoChange = 724,
  kResetSize = 725,
  kDynamic = 726,
  kNonScalingStroke = 727,
  kInternalExtendToZoom = 728,
  kPanX = 729,
  kPanY = 730,
  kPanLeft = 731,
  kPanRight = 732,
  kPanUp = 733,
  kPanDown = 734,
  kManipulation = 735,
  kPinchZoom = 736,
  kLastBaseline = 737,
  kFirstBaseline = 738,
  kFirst = 739,
  kLast = 740,
  kSelfStart = 741,
  kSelfEnd = 742,
  kLegacy = 743,
  kSmooth = 744,
  kScrollPosition = 745,
  kRevert = 746,
  kRevertLayer = 747,
  kUnset = 748,
  kLinearGradient = 749,
  kRadialGradient = 750,
  kConicGradient = 751,
  kRepeatingLinearGradient = 752,
  kRepeatingRadialGradient = 753,
  kRepeatingConicGradient = 754,
  kPaint = 755,
  kWebkitCrossFade = 756,
  kWebkitGradient = 757,
  kWebkitLinearGradient = 758,
  kWebkitRadialGradient = 759,
  kWebkitRepeatingLinearGradient = 760,
  kWebkitRepeatingRadialGradient = 761,
  kWebkitImageSet = 762,
  kImageSet = 763,
  kType = 764,
  kTo = 765,
  kColorStop = 766,
  kRadial = 767,
  kAttr = 768,
  kCounter = 769,
  kCounters = 770,
  kRect = 771,
  kPolygon = 772,
  kFormat = 773,
  kCollection = 774,
  kEmbeddedOpentype = 775,
  kOpentype = 776,
  kSVG = 777,
  kTruetype = 778,
  kWoff = 779,
  kWoff2 = 780,
  kTech = 781,
  kFeaturesOpentype = 782,
  kFeaturesAat = 783,
  kFeaturesGraphite = 784,
  kColorCOLRv0 = 785,
  kColorCOLRv1 = 786,
  kColorSVG = 787,
  kColorSbix = 788,
  kColorCBDT = 789,
  kVariations = 790,
  kPalettes = 791,
  kIncremental = 792,
  kInvert = 793,
  kGrayscale = 794,
  kSepia = 795,
  kSaturate = 796,
  kHueRotate = 797,
  kOpacity = 798,
  kBrightness = 799,
  kContrast = 800,
  kBlur = 801,
  kDropShadow = 802,
  kUrl = 803,
  kRgb = 804,
  kRgba = 805,
  kHsl = 806,
  kHsla = 807,
  kHwb = 808,
  kLab = 809,
  kOklab = 810,
  kLch = 811,
  kOklch = 812,
  kInternalLightDark = 813,
  kSRGBLinear = 814,
  kDisplayP3 = 815,
  kA98Rgb = 816,
  kProphotoRgb = 817,
  kXyz = 818,
  kXyzD50 = 819,
  kXyzD65 = 820,
  kShorter = 821,
  kLonger = 822,
  kDecreasing = 823,
  kIncreasing = 824,
  kIn = 825,
  kColorMix = 826,
  kFrom = 827,
  kR = 828,
  kG = 829,
  kB = 830,
  kH = 831,
  kS = 832,
  kL = 833,
  kW = 834,
  kA = 835,
  kC = 836,
  kMatrix = 837,
  kMatrix3d = 838,
  kPerspective = 839,
  kRotate = 840,
  kRotateX = 841,
  kRotateY = 842,
  kRotateZ = 843,
  kRotate3d = 844,
  kScale = 845,
  kScaleX = 846,
  kScaleY = 847,
  kScaleZ = 848,
  kScale3d = 849,
  kSkew = 850,
  kSkewX = 851,
  kSkewY = 852,
  kTranslate = 853,
  kTranslateX = 854,
  kTranslateY = 855,
  kTranslateZ = 856,
  kTranslate3d = 857,
  kX = 858,
  kY = 859,
  kZ = 860,
  kPath = 861,
  kRay = 862,
  kSides = 863,
  kStrokeBox = 864,
  kCalc = 865,
  kWebkitCalc = 866,
  kMin = 867,
  kMax = 868,
  kClamp = 869,
  kSin = 870,
  kCos = 871,
  kAsin = 872,
  kAtan = 873,
  kAtan2 = 874,
  kAcos = 875,
  kMod = 876,
  kRem = 877,
  kUp = 878,
  kDown = 879,
  kToZero = 880,
  kSign = 881,
  kAbs = 882,
  kPow = 883,
  kSqrt = 884,
  kHypot = 885,
  kLog = 886,
  kExp = 887,
  kInfinity = 888,
  kNegativeInfinity = 889,
  kNan = 890,
  kPi = 891,
  kE = 892,
  kMandatory = 893,
  kProximity = 894,
  kStyle = 895,
  kSize = 896,
  kBlockSize = 897,
  kInlineSize = 898,
  kState = 899,
  kInsetBlockStart = 900,
  kInsetBlockEnd = 901,
  kInsetInlineStart = 902,
  kInsetInlineEnd = 903,
  kAutoFill = 904,
  kAutoFit = 905,
  kVar = 906,
  kInternalVariableValue = 907,
  kEnv = 908,
  kAvoidPage = 909,
  kPage = 910,
  kRecto = 911,
  kVerso = 912,
  kAvoidColumn = 913,
  kP3 = 914,
  kRec2020 = 915,
  kAdd = 916,
  kAutoAdd = 917,
  kTrue = 918,
  kFalse = 919,
  kNoPreference = 920,
  kDark = 921,
  kLight = 922,
  kOnly = 923,
  kReduce = 924,
  kActive = 925,
  kPreserveParentColor = 926,
  kBackButton = 927,
  kFabricated = 928,
  kSelector = 929,
  kContinuous = 930,
  kFolded = 931,
  kStable = 932,
  kBothEdges = 933,
  kMore = 934,
  kLess = 935,
  kCustom = 936,
  kCyclic = 937,
  kSymbolic = 938,
  kNumeric = 939,
  kAdditive = 940,
  kExtends = 941,
  kInternalSimpChineseInformal = 942,
  kInternalSimpChineseFormal = 943,
  kInternalTradChineseInformal = 944,
  kInternalTradChineseFormal = 945,
  kInternalKoreanHangulFormal = 946,
  kInternalKoreanHanjaInformal = 947,
  kInternalKoreanHanjaFormal = 948,
  kInternalHebrew = 949,
  kInternalLowerArmenian = 950,
  kInternalUpperArmenian = 951,
  kInternalEthiopicNumeric = 952,
  kBullets = 953,
  kNumbers = 954,
  kWords = 955,
  kStandard = 956,
  kHigh = 957,
  kConstrainedHigh = 958,
  kLayer = 959,
  kColorContrast = 960,
  kVs = 961,
  kAA = 962,
  kAALarge = 963,
  kAAA = 964,
  kAAALarge = 965,
  kDrop = 966,
  kRaise = 967,
  kXywh = 968,
  kAnchor = 969,
  kAutoSame = 970,
  kAnchorSize = 971,
  kWidth = 972,
  kHeight = 973,
  kSelfBlock = 974,
  kSelfInline = 975,
  kImplicit = 976,
  kEntry = 977,
  kExit = 978,
  kEntryCrossing = 979,
  kExitCrossing = 980,
  kRoot = 981,
  kNearest = 982,
  kSelf = 983,
  kAllowDiscrete = 984,
  kInverted = 985,
  kCrossDocumentSameOrigin = 986,
  kSnap = 987,
  kEnabled = 988,
  kInitialOnly = 989,
};

const int numCSSValueKeywords = 990;
const size_t maxCSSValueKeywordLength = 42;

inline bool IsValidCSSValueID(CSSValueID id)
{
    return id != CSSValueID::kInvalid;
}

CORE_EXPORT const char* getValueName(CSSValueID);
bool isValueAllowedInMode(CSSValueID id, CSSParserMode mode);

}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_
